@@ -5,27 +5,22 @@ use base qw(Helios::Logger);
 use strict;
 use warnings;
 
+use Error qw(:try);
+
 use Helios::LogEntry::Levels qw(:all);
-use Helios::Error::LoggingError;
 
-our $VERSION = '2.61';
-
-# 2011-12-07:  The code of Helios::Service->logMsg() was moved here to 
-# implement internal logging as a Helios::Logger subclass.
-# 2012-01-02:  Re-added support for log_priority_threshold config parameter.
+our $VERSION = '2.40';
 
 =head1 NAME
 
-Helios::Logger::Internal - Helios::Logger subclass implementing Helios internal logging
+Helios::Logger::Internal - Helios::Logger subclass reimplementing Helios internal logging
 
 =head1 SYNOPSIS
 
  #in helios.ini, enable internal Helios logging (this is default)
  internal_logger=on
  
- #in helios.ini, turn off internal logging 
- # make sure you've turned on another logger with the logger= directive
- # otherwise you will have NO logging system active
+ #in helios.ini, turn off internal logging
  internal_logger=off
 
 
@@ -41,15 +36,11 @@ solutions by subclassing Helios::Logger.
 
 =head2 init()
 
-Helios::Logger::Internal->init() attempts to initialize the connection to the 
-Helios collective database so it will be available for later calls to logMsg().
+Helios::Logger::Internal->init() is empty, as an initialization step is unnecessary.
 
 =cut
 
-sub init { 
-	my $self = shift;
-	$self->getDriver();
-}
+sub init { }
 
 
 =head2 logMsg($job, $priority_level, $message)
@@ -65,32 +56,30 @@ sub logMsg {
 	my $level = shift;
 	my $msg = shift;
 
-	my $params   = $self->getConfig();
-	my $jobType  = $self->getJobType();
+	my $params = $self->getConfig();
+	my $jobType = $self->getJobType();
 	my $hostname = $self->getHostname();
 
-# 2012-01-02:  Re-added support for log_priority_threshold config parameter.
     # if this log message's priority is lower than log_priority_threshold,
     # don't bother logging the message
     if ( defined($params->{log_priority_threshold}) &&
         $level > $params->{log_priority_threshold} ) {
         return 1;
     }
-# END 2012-01-02 modification.
 
 	my $retries = 0;
 	my $retry_limit = 3;
 	RETRY: {
-		eval {
+		try{
 			my $driver = $self->getDriver();
 			my $log_entry;
 			if ( defined($job) ) {
 				$log_entry = Helios::LogEntry->new(
 					log_time   => time(),
-					host       => $self->getHostname(),
+					host       => $self->getHostname,
 					process_id => $$,
-					jobid      => $job->getJobid(),
-					funcid     => $job->getFuncid(),
+					jobid      => $job->getJobid,
+					funcid     => $job->getFuncid,
 					job_class  => $jobType,
 					priority   => $level,
 					message    => $msg
@@ -98,7 +87,7 @@ sub logMsg {
 			} else {
 				$log_entry = Helios::LogEntry->new(
 					log_time   => time(),
-					host       => $self->getHostname(),
+					host       => $self->getHostname,
 					process_id => $$,
 					jobid      => undef,
 					funcid     => undef,
@@ -108,11 +97,11 @@ sub logMsg {
 				);
 			}
 			$driver->insert($log_entry);		
-			1;
-		} or do {
-			my $E = $@;
+
+		} otherwise {
+			my $e = shift;
 			if ($retries > $retry_limit) {
-				Helios::Error::LoggingError->throw("$E");
+				throw Helios::Error::DatabaseError($e->text());
 			} else {
 				# we're going to give it another shot
 				$retries++;
@@ -125,6 +114,25 @@ sub logMsg {
 	return 1;
 }
 
+=head1 OTHER METHODS
+
+=head2 getDriver()
+
+Returns a Data::ObjectDriver object for use with the Helios database.
+
+=cut
+
+sub getDriver {
+    my $self = shift;
+    my $config = $self->getConfig();
+    my $driver = Data::ObjectDriver::Driver::DBI->new(
+        dsn      => $config->{dsn},
+        username => $config->{user},
+        password => $config->{password}
+    );  
+    return $driver; 
+}
+
 
 1;
 __END__
