@@ -13,7 +13,6 @@ use Error qw(:try);
 use TheSchwartz;
 use TheSchwartz::Job;
 require XML::Simple;
-$XML::Simple::PREFERRED_PARSER = 'XML::Parser';
 
 use Helios::Error;
 use Helios::Job;
@@ -21,12 +20,15 @@ use Helios::ConfigParam;
 use Helios::LogEntry;
 use Helios::LogEntry::Levels qw(:all);
 
-our $VERSION = '2.22';
+our $VERSION = '2.40';
 
 our $CACHED_CONFIG;
 our $CACHED_CONFIG_RETRIEVAL_COUNT = 0;
 our $WORKER_START_TIME = 0;
 
+our %INIT_LOG_CLASSES;	# for the logging system
+
+
 =head1 NAME
 
 Helios::Service - base class for services in the Helios job processing system
@@ -80,16 +82,22 @@ getConfig() method.  The job's arguments are parsed from XML into a Perl hashref
 available via the job object's getArgs() method.  Then the service object's run() method is 
 called, and is passed the Helios::Job object.
 
-Once the run() method has completed the job and returned, work() determines whether the worker 
-process should exit or stay running.  If the subclass run() method returns a zero and the worker is 
-OVERDRIVE mode, the worker process will stay running, and work() will be called to setup and run 
-another job.  If the run() method returned a nonzero value or the shouldExitOverdrive() returns a 
-true value, the worker process will exit.  If OVERDRIVE mode is disabled, the process will also exit.
+Once the run() method has completed the job and returned, work() determines 
+whether the worker process should exit or stay running.  If OVERDRIVE mode is 
+enabled and the service hasn't been HALTed or told to HOLD, the worker process 
+will stay running, and work() will be called to setup and run another job.  If 
+the service is not in OVERDRIVE mode, the worker process will exit.
+
+=head3 COPYRIGHT
+
+Portions of this method, where noted, are 
+Copyright (C) 2011-2012 by Andrew Johnson.  See the COPYRIGHT AND LICENSE section 
+elsewhere in this document for specific terms.
 
 =cut
 
-sub max_retries { return MaxRetries(); }
-sub retry_delay { return RetryInterval(); }
+sub max_retries { $_[0]->MaxRetries(); }
+sub retry_delay { $_[0]->RetryInterval(); }
 
 sub work {
 	my $class = shift;
@@ -108,10 +116,10 @@ sub work {
         # AND we're not servicing the 10th job (or technically a multiple of ten)
         # THEN just retrieve the pre-existing config
         
-        #[]
-        print "CACHED_CONFIG=",$CACHED_CONFIG,"\n";
-        print "CACHED_CONFIG_RETRIEVAL_COUNT=",$CACHED_CONFIG_RETRIEVAL_COUNT,"\n";
-        
+        if ($self->debug) {
+	        print "CACHED_CONFIG=",$CACHED_CONFIG,"\n";
+	        print "CACHED_CONFIG_RETRIEVAL_COUNT=",$CACHED_CONFIG_RETRIEVAL_COUNT,"\n";
+        }
         if ( defined($CACHED_CONFIG) && 
                 $CACHED_CONFIG->{LAZY_CONFIG_UPDATE} == 1 &&
                 $CACHED_CONFIG->{OVERDRIVE} == 1 &&
@@ -149,30 +157,48 @@ sub work {
 		exit(1);
 	};
 
-	# if this is a meta job, we need to burst it apart 
-	# if it isn't, we should call the service's run() method 
+	# run the job, whether it's a metajob or simple job
 	$self->setJob($job);
-	if ( defined($args->{metajob}) && $args->{metajob} == 1 ) {
-		$self->logMsg($job, LOG_NOTICE, 'Job '.$job->getJobid.' is a metajob.  Bursting...');
-		try {
-			my $numberOfJobs = $self->burstJob($job);
-			$self->logMsg($job, LOG_NOTICE, 'Job '.$job->getJobid.' burst into '.$numberOfJobs.' jobs');
-		} otherwise {
-			my $e = shift;
-			$self->logMsg($job, LOG_ERR, 'Job '.$job->getJobid.' burst failure: '.$e->text);
-		};
-	} else {
-		if ($self->debug) { print "RUNNING JOB\n"; }
-		$return_code = $self->run($job);
-		if ($self->debug) { print "RUN RETURN CODE: ", $return_code,"\n"; }
+# BEGIN CODE Copyright (C) 2011-2012 by Andrew Johnson.
+	eval {
+		if ( $job->isaMetaJob() ) {
+			# metajob
+			if ($self->debug) { print 'CALLING METARUN() for metajob '.$job->getJobid()."...\n"; }
+			$return_code = $self->metarun($job);
+			if ($self->debug) { print 'METARUN() RETURN CODE: '.$return_code."\n"; }
+		} else {
+			# must be a simple job then
+			if ($self->debug) { print 'CALLING RUN() for job '. $job->getJobid()."...\n"; }
+			$return_code = $self->run($job);
+			if ($self->debug) { print 'RUN() RETURN CODE: '. $return_code."\n"; }
+		}
+		1;
+	} or do {
+		my $E = $@;
+		$self->logMsg($job, LOG_CRIT,"Uncaught exception thrown by run() in process ".$$.': '.$E);
+		$self->logMsg($job, LOG_CRIT,'Forcing failure of job '.$job->getJobid().' and exit of process '.$$);
+		$self->failedJob($job, $E, 1);
+		exit(1);
+	};
+
+	# DOWNSHIFT_ON_NONZERO_RUN
+	# previously a nonzero return from run() was taken to mean a failed job, 
+	# and would cause a downshift in OVERDRIVE mode.  This was considered a 
+	# safety feature as it was unknown what caused the job to fail.
+	# But this feature was underdocumented and misunderstood and has been 
+	# removed.  
+	# The new default behavior doesn't pay attention to the value returned
+	# from run() or metarun().  You should mark your job as completed or 
+	# failed in run() or metarun() and not worry about returning anything.
+	# Anyone requiring the old behavior can use the new DOWNSHIFT_ON_NONZERO_RUN
+	# parameter to enable it.
+	if ( defined($self->getConfig()->{DOWNSHIFT_ON_NONZERO_RUN}) &&
+			$self->getConfig()->{DOWNSHIFT_ON_NONZERO_RUN} == 1 && 
+			$return_code != 0
+		) { 
+		exit(1); 
 	}
-
-	# either run() or burstJob() should have marked the job as completed or failed
-	# now we have to decide whether to exit or not
-
-	# if run() returned a nonzero, we're assuming run() ended badly, 
-	# and we need to exit the process to be safe
-	if ($return_code != 0) { exit(1); }
+# END CODE Copyright (C) 2011 by Andrew Johnson.
 
 	# if we're not in OVERDRIVE, the worker process will exit as soon as work() returns anyway 
 	#    (calling shouldExitOverdrive will be a noop)
@@ -182,6 +208,79 @@ sub work {
 		$self->logMsg(LOG_NOTICE,"Class $class exited (downshift)");
 		exit(0);
 	}
+
+	# we'll assume if we got here, things went reasonably well
+	# (run() or metarun() succeeded, or it failed and the errors were caught
+	# we're going to return 0 to the calling routine
+	# in normal mode, this will immediately return to launch_worker() in helios.pl
+	#     (which will exit with this return code)
+	# in OVERDRIVE, this will return to TheSchwartz->work_until_done(), which 
+	# will call this work() with another TheSchwartz::Job, over and over again
+	# until it runs out of jobs.  When the jobs are exhausted, then it returns
+	# to launch_worker() in helios.pl (which then exits with this return code)
+	return 0;
+}
+
+
+=head2 metarun($job)
+
+Given a metajob, the metarun() method runs the job, returning 0 if the 
+metajob was successful and nonzero otherwise.
+
+This is the default metarun() for Helios.  In the default Helios system, 
+metajobs consist of multiple simple jobs.  These jobs are defined in the 
+metajob's argument XML at job submission time.  The metarun() method will 
+burst the metajob apart into its constituent jobs, which are then run by 
+another service.  
+
+Metajobs' primary use in the base Helios system is to speed the job submission 
+process of large job batches.  One metajob containing a batch of thousands of 
+jobs can be submitted and burst apart by the system much faster than thousands 
+of individual jobs can be submitted.  In addition, the faster jobs enter the 
+job queue, the faster Helios workers can be launched to handle them.  If you 
+have thousands (or tens of thousands, or more) of jobs to run, especially if 
+you are running your service in OVERDRIVE mode, you should use metajobs to 
+greatly increase system throughput.
+
+=head3 COPYRIGHT
+
+This method is Copyright (C) 2011-2012 by Andrew Johnson.
+
+See the COPYRIGHT AND LICENSE section elsewhere in this document for specific
+copyright and license terms.
+
+=cut
+
+sub metarun {
+	my $self = shift;
+	my $metajob = shift;
+	my $config = $self->getConfig();
+	my $args = $metajob->getArgs();
+	my $r;
+	
+	eval {
+		$self->logMsg($metajob, LOG_NOTICE, 'Bursting metajob '.$metajob->getJobid);
+		my $jobCount = $self->burstJob($metajob);
+		$self->logMsg($metajob, LOG_NOTICE, 'Metajob '.$metajob->getJobid().' burst into '.$jobCount.' jobs.');
+		1;
+	} or do {
+		my $E = $@;
+		if ( $E->isa('Helios::Error::BaseError') ) {
+			$self->logMsg($metajob, 
+					LOG_ERR, 
+					'Metajob burst failure for metajob '
+					.$metajob->getJobid().': '
+					.$E->text()
+			);
+		} else {
+			$self->logMsg($metajob, 
+					LOG_ERR, 
+					'Metajob burst failure for metajob '
+					.$metajob->getJobid().': '
+					.$E
+			);
+		}
+	};
 }
 
 
@@ -479,6 +578,8 @@ sub getFuncidFromDb {
 }
 
 
+
+
 =head2 jobsWaiting() 
 
 Scans the job queue for jobs that are ready to run.  Returns the number of jobs 
@@ -631,10 +732,14 @@ sub dbConnect {
 }
 
 
-=head2 logMsg([$job,] [$priority,] $msg)
+=head2 logMsg([$job,] [$priority_level,] $message)
 
-Record a message in the Helios log.  In addition to the log message, there are 
-two optional parameters:
+Given a message to log, an optional priority level, and an optional Helios::Job
+object, logMsg() will record the message in the logging systems that have been 
+configured.  The internal Helios logging system is the only system enabled by 
+default.
+
+In addition to the log message, there are two optional parameters:
 
 =over 4
 
@@ -662,148 +767,165 @@ the log priorities defined by Sys::Syslog:
     LOG_DEBUG    7
    
 LOG_DEBUG, LOG_INFO, LOG_NOTICE, LOG_WARNING, and LOG_ERR are the most common 
-used by Helios; LOG_INFO is the default.
+used by Helios itself; LOG_INFO is the default.
 
 =back
 
-As of Helios 2.2, you can specify a logging threshold to better control the 
-logging of your service on-the-fly.  Specifying a 'log_priority_threshold' 
-config parameter in your helios.ini or Ctrl Panel will cause log messages of a 
-lower priority (higher numeric value) will be discarded.  For example, a line 
-in your helios.ini like:
-
- log_priority_threshold=6   
-
-will cause any log messages of priority 7 (LOG_DEBUG) to be discarded.
-
 The host, process id, and service class are automatically recorded with your log 
 message.  If you supplied either a Helios::Job object or a priority level, these
 will also be recorded with your log message.
 
-NOTE: Previous Helios versions also had the ability to log messages to syslogd. 
-This functionality has been removed; see L<HeliosX::ExtLoggerService> and 
-L<HeliosX::Logger::Syslog> if you require this functionality.
+This method returns a true value if successful and throws a 
+Helios::Error::LoggingError if errors occur.   
+
+=head3 LOGGING SYSTEM CONFIGURATION
+
+Several parameters are available to configure Helios logging.  Though these 
+options can be set either in helios.ini or in the Ctrl Panel, it is B<strongly>
+recommended these options only be set in helios.ini.  Changing logging 
+configurations on-the-fly could potentially cause a Helios service (and 
+possibly your whole collective) to become unstable!
+
+The following options can be set in either a [global] section or in an 
+application section of your helios.ini file.
+
+=head4 loggers
+
+ loggers=HeliosX::Logger::Syslog,HeliosX::Logger::Log4perl
+
+A comma delimited list of interface classes to external logging systems.  Each 
+of these classes should implement (or otherwise extend) the Helios::Logger 
+class.  Each class will have its own configuration parameters to 
+set; consult the documentation for the interface class you're trying to 
+configure.
+
+=head4 internal_logger 
+
+ internal_logger=on|off 
+
+Whether to enable the internal Helios logging system as well as the loggers 
+specified with the 'loggers=' line above.  The default is on.  If set to off, 
+the only logging your service will do will be to the external logging systems.
+
+=head4 log_priority_threshold
+
+ log_priority_threshold=1|2|3|4|5|6   
+
+You can specify a logging threshold to better control the 
+logging of your service on-the-fly.  Unlike the above parameters, 
+log_priority_threshold can be safely specified in your Helios Ctrl Panel.  
+Specifying a 'log_priority_threshold' config parameter in your helios.ini or 
+Ctrl Panel will cause log messages of a lower priority (higher numeric value) 
+to be discarded.  For example, a line in your helios.ini like:
+
+ log_priority_threshold=6
+
+will cause any log messages of priority 7 (LOG_DEBUG) to be discarded.
+
+This configuration option is supported by the internal Helios logger 
+(Helios::Logger::Internal).  Other Helios::Logger systems may or may not 
+support it; check the documentation of the logging module you plan to use.
+
+=head3 COPYRIGHT AND LICENSE
+
+The logMsg() method is Copyright (C) 2009 by Andrew Johnson
+
+This method is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself, either Perl version 5.8.0 or,
+at your option, any later version of Perl 5 you may have available.
+
+=head3 WARRANTY
+
+This software comes with no warranty of any kind.
 
 =cut
 
 sub logMsg {
 	my $self = shift;
-	my $priority;
-	my $msg;
+	my @args = @_;
 	my $job;
+	my $level;
+	my $msg;
+	my @loggers;
 
-	# if the first parameter is a Helios::Job object, log extra info
-	if ( ref($_[0]) && $_[0]->isa('Helios::Job') ) {
-		$job = shift;
+	# were we called with 3 params?  ($job, $level, $msg)
+	# 2 params?                      ($level, $msg) or ($job, $msg)
+	# or just 1?                     ($msg)
+
+	# is the first arg is a Helios::Job object?
+	if ( ref($args[0]) && $args[0]->isa('Helios::Job') ) {
+		$job = shift @args;
 	}
 
-	# if we were given 2 params, the first is priority, second message
-	# if only one, it is the message, default to LOG_INFO priority
-	if ( defined($_[0]) && defined($_[1]) ) {
-		$priority = shift;
-		$msg = shift;
+	# if there are 2 params remaining, the first is level, second msg
+	# if only one, it's just the message 
+	if ( defined($args[0]) && defined($args[1]) ) {
+		$level = $args[0];
+		$msg = $args[1];
 	} else {
-		$priority = LOG_INFO;
-		$msg = shift;
+		$level = LOG_INFO;	# default the level to LOG_INFO
+		$msg = $args[0];
 	}
 	
-	my $params = $self->getConfig();
+	my $config = $self->getConfig();
 	my $jobType = $self->getJobType();
 	my $hostname = $self->getHostname();
 
-    # if this log message's priority is lower than log_priority_threshold,
-    # don't bother logging the message
-    if ( defined($params->{log_priority_threshold}) &&
-        $priority > $params->{log_priority_threshold} ) {
-        return 1;
+    # grab the names of all the configured loggers to try
+    if ( defined($config->{loggers}) ) {
+	    @loggers = split(/,/, $config->{loggers});
+    }
+    
+    # inject the internal logger automatically
+    # UNLESS it has been specifically turned off
+    unless ( defined($config->{internal_logger}) && 
+        ( $config->{internal_logger} eq 'off' || $config->{internal_logger} eq '0') ) {
+    	unshift(@loggers, 'Helios::Logger::Internal');
     }
 
-    # Sys::Syslog support
-	# log the message to syslog IF log_facility was set
-	#[]? removed in favor of HeliosX::Logger::Syslog
-=disable
-	if ( defined($params->{syslog_facility}) ) {
-        unless ( 'Sys::Syslog'->can('openlog') ) {
-            eval "require Sys::Syslog";
-            throw Helios::Error::Fatal("Unable to load Sys::Syslog: ".$@) if $@;
-        }		
-		openlog($jobType, $params->{syslog_options}, $params->{syslog_facility});
-		syslog($priority, $msg);
-		closelog();
-	}
-=cut
-
-	# log to database
-	my $retries = 0;
-	my $retry_limit = 3;
-	RETRY: {
-		try{
-			my $driver = $self->getDriver();
-			my $log_entry;
-			if ( defined($job) ) {
-				$log_entry = Helios::LogEntry->new(
-					log_time   => time(),
-					host       => $self->getHostname,
-					process_id => $$,
-					jobid      => $job->getJobid,
-					funcid     => $job->getFuncid,
-					job_class  => $jobType,
-					priority   => $priority,
-					message    => $msg
-				);
-			} else {
-				$log_entry = Helios::LogEntry->new(
-					log_time   => time(),
-					host       => $self->getHostname,
-					process_id => $$,
-					jobid      => undef,
-					funcid     => undef,
-					job_class  => $jobType,
-					priority   => $priority,
-					message    => $msg
-				);
+	# if no loggers configured, this whole section will be skipped
+	foreach my $logger (@loggers) {
+		# init the logger if it hasn't been initialized yet
+		unless ( defined($INIT_LOG_CLASSES{$logger}) ) {
+			if ( $logger !~ /^[A-Za-z]([A-Za-z0-9_\-]|:{2})*[A-Za-z0-9_\-]$/ ) {
+				Helios::Error::LoggingError->throw("Sorry, requested Logger name is invalid: ".$logger);
 			}
-			$driver->insert($log_entry);		
-
+			# attempt to init the class
+			unless ( $logger->can('init') ) {
+		        eval "require $logger";
+		        throw Helios::Error::LoggingError($@) if $@;
+			}
+			$logger->setConfig($config);
+			$logger->setJobType($jobType);
+			$logger->setHostname($hostname);
+            try {
+    			$logger->init();
+            } otherwise {
+            	# our only resort is to use the internal logger
+            	my $e = shift;
+            	print $e->text(),"\n"; #[]
+                Helios::Logger::Internal->setConfig($config);
+                Helios::Logger::Internal->setJobType($jobType);
+                Helios::Logger::Internal->setHostname($hostname);
+                Helios::Logger::Internal->init();
+            	Helios::Logger::Internal->logMsg(undef, LOG_EMERG, $logger.' CONFIGURATION ERROR: '.$e->text());
+            };
+			$INIT_LOG_CLASSES{$logger} = $logger;
+		}
+		try {
+		  $logger->logMsg($job, $level, $msg);
 		} otherwise {
-			my $e = shift;
-			if ($retries > $retry_limit) {
-				throw Helios::Error::DatabaseError($e->text());
-			} else {
-				# we're going to give it another shot
-				$retries++;
-				sleep 10;
-				next RETRY;
-			}
-		};
-	} # retry block end
-	return 1;
-}
-
-
-=head2 parseArgXML($xml) [DEPRECATED]
-
-Given a string of XML, parse it into a mixed hash/arrayref structure.  This uses
-XML::Simple.
-
-This method is DEPRECATED; its functionality has been moved to 
-Helios::Job->parseArgXML().  You should use the Helios::Service->getJobArgs() 
-instead of this method.
-
-=cut
-
-sub parseArgXML {
-	my $self = shift;
-	my $xml = shift;
-
-	my $xs = XML::Simple->new(SuppressEmpty => undef);
-	my $args;
-	try {
-		$args = $xs->XMLin($xml);
-	} otherwise {
-		throw Helios::Error::InvalidArg($!);
-	};
-	return $args;
+            my $e = shift;
+            print $e->text(),"\n"; #[]
+            Helios::Logger::Internal->setConfig($config);
+            Helios::Logger::Internal->setJobType($jobType);
+            Helios::Logger::Internal->setHostname($hostname);
+            Helios::Logger::Internal->init();
+            Helios::Logger::Internal->logMsg(undef, LOG_EMERG, $logger.' LOGGING FAILURE: '.$e->text());
+		};			
+	}
+	
+	return 1;	
 }
 
 
@@ -893,7 +1015,7 @@ sub burstJob {
 These are the basic methods that define your Helios service.  The run() method 
 is the only one required. 
 
-=head2 run()
+=head2 run($job)
 
 This is a default run method for class completeness.  You have to override it 
 in your own Helios service class. 
@@ -947,11 +1069,17 @@ L<TheSchwartz>, L<XML::Simple>, L<Config::IniFiles>, L<Sys::Syslog>
 
 =head1 AUTHOR
 
-Andrew Johnson, E<lt>lajandy at cpan dotorgE<gt>
+Andrew Johnson, E<lt>lajandy at cpan dot orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2008-9 by CEB Toolbox, Inc.
+Copyright (C) 2008-9 by CEB Toolbox, Inc., except as noted.
+
+Portions of this software, where noted, are 
+Copyright (C) 2009 by Andrew Johnson.
+
+Portions of this software, where noted, are
+Copyright (C) 2011-2012 by Andrew Johnson.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.0 or,
